package HSPC::PluginToolkit::DM::FileExchange;

use strict;
use SWsoft::protect;

use HSPC::MT::PluginTools qw(get_request_data);
use HSPC::Logger qw(sw_log_warn);
use HSPC::WebSystemLib qw(sw_uriparam);
use HSPC::ParamsValidate qw(:DEFAULT);
use HSPC::PluginToolkit::General qw(upload_file);

use constant BASE_FILEPATH => '/var/opt/hspc-root/template/HSPC/MT/Plugin/DM/';
use constant DEFAULT_FILEPATH => 'default/';

use base qw(Exporter);
our @EXPORT_OK = qw(get_fullpath upload_file local_load_file remove_file create_html_link);

sub load_file {
	my $class = shift;

	my $filename = sw_uriparam('filename');
	my $filetype = sw_uriparam('filetype') || 'text/rtf';
	my $template_id = sw_uriparam('template_id');
	my $plugin_id = sw_uriparam('plugin_id');
	my $vendor_id = sw_uriparam('doc_id');

	my $fullpath = $class->_get_fullpath(
		plugin_template_id => $template_id,
		plugin_id => $plugin_id,
		vendor_id => $vendor_id,
		file_name => $filename
	);
	if(!$fullpath){
		sw_log_warn("Requested file not found.");
		return {status => 'ERROR', error => 'Requested file not found.'};
	}

	my $r = Apache->request;
	local $| = 1;
	$r->status(200);
	my $attachment_type = ($ENV{system_obj}->browser->ie) ?
		'inline' : 'attachment';

	$r->header_out('Content-Disposition',
		$attachment_type . ';filename="'. $filename .'"');
	$r->send_http_header($filetype);
	my $error_flag;
	open(IN, "<$fullpath") || ($error_flag = 1);
	if($error_flag){
		sw_log_warn("Can't open file: $fullpath $!");
		return {status => 'POSTED'};
	}

	local $/ = undef;
	my $file_content = <IN>;
	print STDOUT $file_content;
	local $/ = "\n";
	close IN;

	return {status => 'POSTED'};
}

sub upload_file {
	my %h = (
		view_name => undef,
		file_name => undef,
		is_image => 0,
		overwrite => 1,
		@_
	);

	my $basepath_result = __PACKAGE__->_get_own_basepath();
	if(!$basepath_result->{is_success}){
		return {
			is_success => $basepath_result->{is_success},
			message => $basepath_result->{message}
		}
	}
	my $basepath = $basepath_result->{basepath};

	my $result = HSPC::PluginToolkit::General::upload_file(
		view_name => $h{view_name},
		base_path => $basepath,
		file_name => $h{file_name},
		is_image => $h{is_image},
		overwrite => $h{overwrite}
	);

	if(ref($result) ne 'HASH'){
		sw_log_warn("Upload is failed: " . $result);
		return {
			is_success => 0,
			message => "Upload is failed: " . $result
		};
	} else {
		chmod($basepath . $h{file_name}, '06777');
		return {
			is_success => 1,
			upload_result => $result
		};
	}
}

sub local_load_file {
	my %h = (
		file_name => undef,
		@_
	);

	my $file_data;
	my $fullpath = get_fullpath(file_name => $h{file_name});
	if(!$fullpath){
		sw_log_warn("Requested local file not found.");
		return {
			is_success => 0,
			description => "Requested local file not found."
		}
	}
	open IN,"<$fullpath" || return {
		is_success => 0,
		description => "error open file $fullpath ($!)"
	};

	$/ = undef;
	$file_data = <IN>;
	$/ = "\n";
	close IN;

	my $result;
	if($file_data){
		$result = {
			is_success => 1,
			description => "File is loaded: " . $fullpath,
			file_data	=> "$file_data"
		};
	} else {
		$result = {
			is_success => 0,
			description => "File isn't loaded: " . $fullpath
		};
	}
	return $result;
}

sub get_fullpath {
	my %h = (
		file_name => undef,
		@_
	);
	my $plugin_template_id = get_request_data('plugin_template_id');
	my $plugin_id = get_request_data('plugin_config_id');
	my $vendor_id = get_request_data('plugin_vendor_id');
	return __PACKAGE__->_get_fullpath(
		plugin_template_id => $plugin_template_id,
		plugin_id => $plugin_id,
		vendor_id => $vendor_id,
		file_name => $h{file_name}
	);
}

sub remove_file {
	my %h = (
		file_name => undef,
		@_
	);
	my $basepath_result = __PACKAGE__->_get_own_basepath();
	if(!$basepath_result->{is_success}){
		return {
			is_success => $basepath_result->{is_success},
			message => $basepath_result->{message}
		}
	}
	my $basepath = $basepath_result->{basepath};
	my $fullpath = $basepath . $h{file_name};
	if(!$fullpath){
		sw_log_warn("Requested local file not found.");
		return {
			is_success => 0,
			description => "Requested local file not found."
		}
	}
	unlink($fullpath);
	return {
		is_success => 1
	};
}

sub create_html_link {
	my %h = (
		file_name => undef,
		@_
	);
	my $filename = $h{file_name};
	my $plugin_template_id = get_request_data('plugin_template_id');
	my $plugin_id = get_request_data('plugin_config_id');
	my $vendor_id = get_request_data('plugin_vendor_id');

	return $ENV{COMPREP}->link(
		alias => 'domain_plugin_toolkit_load_file',
		[
			filename => $h{file_name},
			template_id => $plugin_template_id,
			plugin_id => $plugin_id,
			doc_id => $vendor_id
		]
	);
}

sub _get_fullpath {
	my $class = shift;
	my %h = (
		plugin_template_id => undef,
		plugin_id => undef,
		vendor_id => undef,
		file_name => undef,
		@_
	);
	my $plugin_template_id = $h{plugin_template_id};
	my $plugin_id = $h{plugin_id};
	my $vendor_id = $h{vendor_id};
	my $filename = $h{file_name};

	if(!$plugin_template_id || !$plugin_id || !$vendor_id){
		sw_log_warn("Plugin, template and vendor IDs should be specified. ('" .
			$plugin_template_id . "', '" . $plugin_id . "', '" . $vendor_id . "')");
		return undef;
	}

	my $fullpath = &BASE_FILEPATH . $plugin_template_id . '/' . $plugin_id .
		'/' . $vendor_id . '/' . $filename;
	$fullpath = &BASE_FILEPATH . $plugin_template_id . '/' . $plugin_id .
		'/' . &DEFAULT_FILEPATH . $filename if(!-f($fullpath));
	$fullpath = &BASE_FILEPATH . $plugin_template_id . '/' . &DEFAULT_FILEPATH .
		$filename if(!-f($fullpath));
	$fullpath = undef if(!-f($fullpath));

	return $fullpath;
}

sub _get_own_basepath {
	my $class = shift;

	my $plugin_template_id = get_request_data('plugin_template_id');
	my $plugin_id = get_request_data('plugin_config_id');
	my $vendor_id = get_request_data('plugin_vendor_id');

	my $basepath = &BASE_FILEPATH;
	if(!$plugin_template_id || !$plugin_id || !$vendor_id){
		sw_log_warn("Environment is corrupted. ('" .
			$plugin_template_id . "', '" . $plugin_id . "', '" . $vendor_id . "')");
		return {
			is_success => 0,
			message => 'Environment is corrupted.'
		};
	} else {
		$basepath .= $plugin_template_id . '/' . $plugin_id . '/' . $vendor_id . '/';
		return {
			is_success => 1,
			basepath => $basepath
		};
	}
}

1;
