package HSPC::PluginWizard::LM;


use strict;
use SWsoft::protect;

use base qw(HSPC::PluginWizard::Abstract);

use HSPC::Logger qw(sw_die sw_log sw_log_warn);
use HSPC::MT::PluginTools qw(set_request_data get_request_data);
use HSPC::WebSystemLib;
use HSPC::WebDB qw(atrans);
use Error qw(:try);
use HSPC::MT::PluginConfig::LM;

my @STEPS = (
	{title_id => 'general_info_uc', code => 'gen'},
	{title_id => 'plugin_conf_uc',  code => 'plugin'},
);


sub new {
	my $class = shift;

	my $obj = $class->SUPER::new(@_);

	if ($obj) {
		push @{$obj->{wz_steps}}, @STEPS;
	}
	return $obj;
}##/new

sub edit_teaser {
	my ($class,%h) = @_;
	my $class = shift;
	my %h = (
		page     => undef,
		start    => undef,
		@_
	);

	if ($h{start}) {
		my $obj = HSPC::MT::PluginConfig::LM->new();
		$obj->plugin_type("LM");
		$obj->vendor_id( $ENV{security_obj}->account_no() );
		$h{created_object} = $obj;
	}
	$class->SUPER::edit_teaser(%h);
}

sub teaser_step_gen {
	my $self = shift;
	
	my $page = $self->page();
	$page->title(pages => []);
	$self->form_open();
	
	$page->edit_view_input(
		title_id   => 'plugin_name_uc',
		view_name  => 'config_name',
		not_empty  => 1,
		obj_attr   => 'name',
		object     => $self->{obj},
		max_length => 55,
	);

	my $templates = HSPC::MT::PluginConfig::LM->find_templates(
		plugin_type => $self->{obj}->plugin_type(),
		vendor_id   => $self->{obj}->vendor_id(),
	);
	$page->edit_view_combo(
		title_id  => 'plugin_template_uc',
		view_name => 'template_id',
		options   => [
			sort {uc($a->[1]) cmp uc($b->[1])}
				map{[$_,HSPC::MT::PluginConfig::LM->find_mt_template(
							template_id => $_,
							plugin_type => $self->{obj}->plugin_type,
							)->get_title()]}@{$templates}],
		not_empty     => 1,
		options_plain => 1,
		value         => 0,
	);

	$self->form_close(edit_btns=>'save',new_btns=>'back_next');

	return {STATUS => 'OK'};
}##/teaser_step_gen


sub teaser_step_plugin {
	my $self = shift;
	my $page = $self->page();

	$page->title( pages => [] );
	$self->form_open();
	my $pr_template = $self->{obj}->find_presentation_template();
	my $content;
	eval {
		my $config_data = $self->{obj}->config_data() || {};
		$content = $pr_template->edit_form(
			config => $config_data,page=>$page
		);
	};
	if ($@) {
		sw_log_warn("Error load presentation form: " . $@);
	}
	$page->edit_append( content => $content );
	$self->form_close(new_btns=>'cancel_back_finish');

	return {STATUS => 'OK'};
}


sub process_step_plugin {
	my $self = shift;
	
	my $obj = $self->{obj};
	my $pr_template = $obj->find_presentation_template();

	set_request_data('plugin_config_id', $obj->id());
	set_request_data('plugin_vendor_id', $obj->vendor_id());
	set_request_data('plugin_template_id', $obj->template_id());
	
	eval "use $pr_template;";

	my $skip_check = 0;
	$skip_check = 1 if ( sw_argparam('btn_back') );

	my $collect_result = $self->{obj}->config_data();
	my $error;
	try {
		$collect_result = $pr_template->collect_data(
			config => $self->{obj}->config_data(),
			skip_check=>$skip_check,plugin_id=>$obj->id()
		);
	} catch HSPC::Exception::Plugin::GeneralError with {
		my $e = shift;
		$error = $e->log_message();
	} catch HSPC::Exception::Plugin::InvalidParameter with {
		my $e = shift;
		$error = $e->message;
	} otherwise {
		$error = shift;
	};
	if ( $error ) {
		sw_log_warn("License plugin configuration error: $error");
		$self->add_error(
			string => $error
		);
		return {step_inc => 0};
	}
	$self->{obj}->config_data($collect_result);

	return {STATUS=>'PROCESS'};
}##/process_step_plugin


sub handle_finish {
	my $self = shift;

	my $trans = atrans();
	my $result = $self->{obj}->save();

	set_request_data('plugin_config_id', $self->{obj}->id() );
	set_request_data('plugin_vendor_id', $self->{obj}->vendor_id() );

	my ($error, $config);

	try {
		my $pr_template = $self->{obj}->find_presentation_template();
		$config = $pr_template->collect_data( config => $self->{obj}->config_data() );
	} catch HSPC::Exception::Plugin::GeneralError with {
		my $e = shift;
		$error = $e->log_message;
		sw_log_warn($error);
		$error = $e->vendor_message;
	} catch HSPC::Exception::Plugin::InvalidParameter with {
		my $e = shift;
		$error = $e->message;
	} otherwise {
		$error = shift;
	};
	
	if ($error) {
		$self->{obj}->id(0);
		$trans->rollback();
		$self->add_error( string => "$error" );
		return {step_inc => 0};
	}

	$self->{obj}->config_data($config);
	$self->{obj}->is_active(0);
	$self->{obj}->save();
	$trans->commit();
	## add task 
	my $mt_template = $self->{obj}->find_mt_template();
	$mt_template->post_config_actions(
		plugin_id=>$self->{obj}->id(),
		vendor_id=>$self->{obj}->vendor_id(),
		async => 1,
	);
	
	$self->{free_session} = 1;
	## todo was scheduled
	return $self->redirect(sl => 'configuration_added_ok');
}##/handle_finish


sub redirect {
	my ($self,%h) = @_;

	return sw_web_redirect_comprep(alias => 'license_plugins');
}


1;
